const React = require('react');

const flushAllPromises = () => new Promise((resolve) => setImmediate(resolve));

let mockAuthState = {};
const getMockAuthState = () => mockAuthState;
const setMockAuthState = (state) => mockAuthState = state;

const mockUseAuthorizationTools = {
  cancelPasswordChange: jest.fn(),
  cancelPasswordReset: jest.fn(),
  completeForcedPasswordChange: jest.fn(),
  completePasswordChange: jest.fn(),
  completePasswordReset: jest.fn(),
  getSession: jest.fn(),
  getUser: jest.fn(),
  refreshSession: jest.fn(),
  requestPasswordReset: jest.fn(),
  signIn: jest.fn(),
  signOut: jest.fn(),
  startPasswordChange: jest.fn(),
  startPasswordReset: jest.fn(),
  verifySession: jest.fn()
};

let mockConfig = {};
const getMockConfig = () => mockConfig;
const setMockConfig = (config) => mockConfig = config;

let mockDispatchState;
const getMockDispatchState = () => mockDispatchState;
const setMockDispatchState = (state) => mockDispatchState = state;
const mockDispatch = jest.fn().mockImplementation((actionResult) => {
  if (typeof actionResult === 'function') {
    actionResult(mockDispatch, mockDispatchState);
  }
});

let mockLocalizationAvailableLocales = ['en'];
const getMockLocalizationAvailableLocales = () => mockLocalizationAvailableLocales;
const setMockLocalizationAvailableLocales = (locales) => mockLocalizationAvailableLocales = locales;

let mockLocalizationLocale = 'en';
const getMockLocalizationLocale = () => mockLocalizationLocale;
const setMockLocalizationLocale = (locale) => mockLocalizationLocale = locale;

const mockStringsFormat = jest.fn().mockImplementation((key, values) => (
  `Formatted string for key "${key}" with values {${Object.keys(values).map((k) => ` ${k}: "${values[k]}"`).join(',')} }`
));

let mockLocalizationStrings = { format: mockStringsFormat };
const getMockLocalizationStrings = () => mockLocalizationStrings;
const setMockLocalizationStrings = (strings) => mockLocalizationStrings = {
  ...strings,
  format: mockStringsFormat
};

let mockFetchJsonSuccess = true;
const getMockFetchJsonSuccess = () => mockFetchJsonSuccess;
const setMockFetchJsonSuccess = (success) => mockFetchJsonSuccess = success;

let mockFetchJsonResponse;
const getMockFetchJsonResponse = () => mockFetchJsonResponse;
const setMockFetchJsonResponse = (res) => mockFetchJsonResponse = res;

let mockServiceResponse = { body: { id: '1', name: 'Hogwarts' }, success: true };
const getMockServiceResponse = () => mockServiceResponse;
const setMockServiceResponse = (res) => mockServiceResponse = res;

let mockThemeBrand = 'wjec';
const getMockThemeBrand = () => mockThemeBrand;
const setMockThemeBrand = (brand) => mockThemeBrand = brand;

let mockThemeColorPalette = {};
const getMockThemeColorPalette = () => mockThemeColorPalette;
const setMockThemeColorPalette = (colorPalette) => mockThemeColorPalette = colorPalette;

const deepCopy = (obj) => {
  if (typeof obj !== 'object' || obj === null) {
    return obj;
  }

  const copy = Array.isArray(obj) ? [] : {};

  for (const key in obj) {
    const val = obj[key];

    copy[key] = deepCopy(val);
  }

  return copy;
};

let contextMocks = [];
const clearContextMocks = () => contextMocks = [];
const getContextMocks = () => contextMocks;

const createMockContext = (initialState) => {
  let state = deepCopy(initialState);
  const dispatch = jest.fn();
  const hook = jest.fn().mockImplementation(() => [
    mockContext.state,
    dispatch
  ]);

  const Provider = ({ children }) => React.createElement(React.Fragment, {}, children);

  const mockContext = {
    dispatch,
    hook,
    Provider,
    state
  };

  const resetState = () => {
    state = deepCopy(initialState);

    for (const key in mockContext.state) {
      mockContext.state[key] = state[key];
    }
  };

  contextMocks.push({ dispatch, resetState });

  return mockContext;
};

module.exports = {
  flushAllPromises,

  getMockAuthState,
  setMockAuthState,
  mockUseAuthorizationTools,

  getMockConfig,
  setMockConfig,

  mockDispatch,
  getMockDispatchState,
  setMockDispatchState,

  getMockLocalizationAvailableLocales,
  setMockLocalizationAvailableLocales,

  getMockLocalizationLocale,
  setMockLocalizationLocale,

  mockStringsFormat,

  getMockLocalizationStrings,
  setMockLocalizationStrings,

  getMockFetchJsonSuccess,
  setMockFetchJsonSuccess,

  getMockFetchJsonResponse,
  setMockFetchJsonResponse,

  getMockServiceResponse,
  setMockServiceResponse,

  getMockThemeBrand,
  setMockThemeBrand,

  getMockThemeColorPalette,
  setMockThemeColorPalette,

  createMockContext,
  clearContextMocks,
  getContextMocks
};
